@@ -1,5 +1,5 @@
 /* xgettext glade backend.
-   Copyright (C) 2002-2003, 2006 Free Software Foundation, Inc.
+   Copyright (C) 2002-2003, 2006, 2015 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2002.
 
    This program is free software: you can redistribute it and/or modify
@@ -27,25 +27,16 @@ extern "C" {
 #endif
 
 
-#define EXTENSIONS_GLADE \
-  { "glade",     "glade"    },                                          \
-  { "glade2",    "glade"    },                                          \
-  { "ui",        "glade"    },                                          \
+/* The scanner is implemented as ITS rules, in its/glade[12].its and
+   its/gtkbuilder.its.  */
+
+#define EXTENSIONS_GLADE                                             \
+  { "glade",     NULL    },                                          \
+  { "glade2",    NULL    },                                          \
+  { "ui",        NULL    },                                          \
 
 #define SCANNERS_GLADE \
-  { "glade",            extract_glade, NULL, NULL, NULL, NULL },              \
-
-/* Scan a glade XML file and add its translatable strings to mdlp.  */
-extern void extract_glade (FILE *fp, const char *real_filename,
-                           const char *logical_filename,
-                           flag_context_list_table_ty *flag_table,
-                           msgdomain_list_ty *mdlp);
-
-
-/* Handling of options specific to this language.  */
-
-extern void x_glade_extract_all (void);
-extern void x_glade_keyword (const char *name);
+  { "glade",            NULL, NULL, NULL, NULL, NULL },              \
 
 
 #ifdef __cplusplus
